@@ -13,8 +13,8 @@
 //! <br>
 //!
 //! Huh?
-//! - Imagine you are writing this cool API and want the world to have it.
-//! - The world, however, is running Unity, C, Python, ... all at the same time.
+//! - Imagine you are writing this cool API and want everyone to use it.
+//! - Everyone else, however, is running Unity, C, Python, ...
 //! - "Not a problem", you say, "I'll just use Interoptopus".
 //!
 //! And you'll live happily<sup>*</sup> ever after.
@@ -39,7 +39,7 @@
 //!     println!("{}", input.x);
 //! }
 //!
-//! inventory!(ffi_inventory, [], [my_function], []);
+//! inventory!(ffi_inventory, [], [my_function], [], []);
 //!
 //! ```
 //!
@@ -89,10 +89,11 @@
 //!
 //! If you **need API design guidance** the following (wip) [**C# call-cost table**](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_csharp/benches/BENCHMARK_RESULTS.md)<sup>🔥</sup> can help.
 //!
-//! ## Current Status
+//! ## Changelog
 //!
-//! - June 20, 2021 - Alpha. Has generated simple working<sup>TM</sup> bindings for a few projects for a week now, many things missing.
-//! - June 13, 2021 - Pre-alpha. Has generated C#, C, Python-CFFI bindings at least once, many things missing, untested.
+//! - **v0.3** - Better compatibility with generics.
+//! - **v0.2** - Introduced "patterns"; produces generally _working_ interop for C#.
+//! - **v0.1** - Has generated C#, C, Python-CFFI bindings at least once.
 //!
 //!
 //! ## FAQ
@@ -105,11 +106,10 @@
 //!
 //! - Bug fixes can be submitted directly. Major changes should be filed as issues
 //! first.
-//!
-//! - Anything that would make previously working bindings change behavior or stop compiling
-//! is a major change; which doesn't mean we're opposed to breaking stuff before 1.0, just that
+//! - Anything that makes previously working bindings change behavior or stop compiling
+//! is a major change;
+//! - This doesn't mean we're opposed to breaking stuff just that
 //! we'd like to talk about it before it happens.
-//!
 //! - New features or patterns must be materialized in the reference project and accompanied by
 //! an interop test (i.e., a backend test running C# / Python against a DLL invoking that code)
 //! in at least one included backend.
@@ -161,20 +161,21 @@ pub mod lang {
 ///
 /// # Usage
 ///
-/// This macro must be invoked with exactly 4 parameters:
+/// This macro must be invoked with exactly 5 parameters:
 ///
 /// ```ignore
 /// # use interoptopus::inventory_function;
-/// inventory_function!(symbol, consts, functions, patterns);
+/// inventory_function!(symbol, consts, functions, extra_types, patterns);
 /// ```
 ///
 /// Where
 /// - `symbol` - the name of the exported inventory function producing a [`Library`],
 /// - `consts` - a list of [`#[ffi_constant]`](crate::ffi_constant) constants to include `[C1, C2, ...]`,
 /// - `functions` - a list of [`#[ffi_function]`](crate::ffi_function) functions to include `[f1, f2, ...]`,
+/// - `extra_types` - additional types not inferred from `functions`, e.g., when using `void` pointers.
 /// - `patterns` - a list of [`LibraryPattern`](crate::patterns::LibraryPattern) to include `[p1, ...]`,
 ///
-/// Any of `consts`, `functions` or `patters` can be an empty list `[]` instead. Types are always
+/// Any of `consts`, `functions` or `patters` can be an empty list `[]` instead. Most types are
 /// inferred automatically based on the used functions.
 ///
 /// # Example
@@ -193,7 +194,7 @@ pub mod lang {
 ///     my_inventory_function,
 ///     [ MY_CONSTANT ],
 ///     [ f ],
-///     []
+///     [], []
 /// );
 /// ```
 ///
@@ -217,6 +218,11 @@ macro_rules! inventory {
         ),*
         ],
         [
+        $(
+            $extra_type:ty
+        ),*
+        ],
+        [
         $(
             $pattern:path
         ),*
@@ -227,7 +233,7 @@ macro_rules! inventory {
             use $crate::lang::rust::FunctionInfo;
             use $crate::lang::rust::ConstantInfo;
 
-            let mut constants: Vec<$crate::lang::c::Constant> = Vec::new();
+            let mut constants: ::std::vec::Vec<$crate::lang::c::Constant> = ::std::vec::Vec::new();
             $(
                 {
                     use $const as user_constant;
@@ -235,7 +241,7 @@ macro_rules! inventory {
                 }
             )*
 
-            let mut functions: Vec<$crate::lang::c::Function> = Vec::new();
+            let mut functions: ::std::vec::Vec<$crate::lang::c::Function> = ::std::vec::Vec::new();
             $(
                 {
                     use $function as user_function;
@@ -243,7 +249,15 @@ macro_rules! inventory {
                 }
             )*
 
-            let mut patterns: Vec<$crate::patterns::LibraryPattern> = Vec::new();
+            let mut extra_types: ::std::vec::Vec<$crate::lang::c::CType> = ::std::vec::Vec::new();
+            $(
+                {
+                    extra_types.push(< $extra_type as $crate::lang::rust::CTypeInfo >::type_info() );
+                }
+            )*
+
+
+            let mut patterns: ::std::vec::Vec<$crate::patterns::LibraryPattern> = ::std::vec::Vec::new();
             $(
                 {
                     let pattern: $crate::patterns::LibraryPattern = $pattern().into();
@@ -260,7 +274,8 @@ macro_rules! inventory {
                 }
             )*
 
-            $crate::Library::new(functions, constants, patterns)
+
+            $crate::Library::new(functions, constants, patterns, extra_types)
         }
     };
 }
